#ProcessorUtils.py
'''Functions called by Processor.py and ProcessorComponents.py . '''

import glob
import json
import logging
import re
import subprocess
import sys

# define logging filter to obscure ODK passwords

def filter_exception_message(exc_info):
    '''To mask out passwords from subprocess input list.'''

    error = exc_info[1]

    # subprocess input appears as a list
    if len(error.args) > 1:
        if isinstance(error.args[1],list):

            msg_list = error.args[1]

            pwd_arg = '--odk_password'
            
            if pwd_arg in msg_list:
                
                # get the index of the password, it comes after the input flag
                passwd_flag_idx = [i for i,val in enumerate(msg_list) if val==pwd_arg][0]
                passwd_idx = passwd_flag_idx + 1

                # replace the password
                msg_list[passwd_idx] = '***'
    
    return

class PasswordODKFilter(logging.Filter):
    '''Built with 
    https://docs.python.org/3.6/howto/logging-cookbook.html#configuring-filters-with-dictconfig
    '''

    def __init__(self, param=None):
        # example of how to pass parameters
        self.param = param
    
    def filter(self, record):

        # on the primary message
        input = record.msg
        masked_msg = re.sub("password', '\S+'","password', '***'",input)
        record.msg = masked_msg

        # on any exception info
        if record.exc_info is not None:

            filter_exception_message(record.exc_info)

        # return boolean to determine whether record gets logged
        return True


def add_filters_to_sublogger(logger_case):

    for h in logger_case.handlers:

        h.addFilter(PasswordODKFilter())


logger = logging.getLogger('Processor.Utils')
add_filters_to_sublogger(logger)

def open_and_check_config(configFile):
    logger.info("Loading config")

    try:
        with open(configFile) as config_file:
            config = json.load(config_file)
    except:
        logger.exception(f"Failure opening config file {configFile}")
        raise

    logger.debug(json.dumps(config,indent=2))

    #TODO: Checking config file for required formatting
    try:
        pass
    except AssertionError as e:
        logger.exception("Configuration file is not formatted as required")
        raise

    return config

def get_only_existing_globs(file_globs,inplace=True):
    
    if inplace:
        for i,fg in enumerate(file_globs):
            if not glob.glob(fg):
                logger.warning(f'This file pattern returns nothing and therefore does not contribute to result:\n{fg}')
                file_globs.pop(i)
    else:
        globs_out = []
        for i,fg in enumerate(file_globs):
            if not glob.glob(fg):
                logger.warning(f'This file pattern returns nothing and therefore does not contribute to result:\n{fg}')
                continue
            globs_out += [fg]
        return globs_out

def subprocess_and_log(cmd,description_short,description_long,check=True,log_type='error',**kwargs):
    '''Run a shell command (described by a comma separated list) and send stdout
    and stderr to logfile, and raise any exception.
    
    log_type is a string to determine which log message level to use, must be 
    one of the logging options: info, debug, warn, warning, error, exception or
    fatal.'''

    # determine message type to write to log
    log_call = getattr(logger,log_type)

    try:
        process = subprocess.run(
                cmd,
                check=check,
                stdout = subprocess.PIPE,
                stderr = subprocess.STDOUT,
                **kwargs)

        for line in process.stdout.decode('utf-8').split(r'\n'):
            logger.info(f"{description_short} : " + line)

    except subprocess.CalledProcessError as e:

        for line in e.stdout.decode('utf-8').split(r'\n'):
            logger.info(f"{description_short} : " + line)

        log_call(f"Some failure when running {description_long}", exc_info=True)
        raise

    return process

def endScript(premature=True):

    if not premature:

        logger.info(f'Script finished!')

    else:

        logger.info(f'Script finished prematurely')

    logger.info(f'--------')

    sys.exit()

def endJob(status,ignore_inprogress=False,**kwargs):

    # get status file fresh, and ensure one exists
    status.status = status.get()

    assert status.has_status

    if status.is_inprogress() & (ignore_inprogress == False):

        logger.warning(f'Workflow problem: status is {status.status} but endJob called, forcing error status')

        status.reset('ERROR')

    endScript(**kwargs)
